/**
 * @link http://stackoverflow.com/questions/24471109/recyclerview-onclick/26196831#26196831
 */
abstract public class AbstractRecyclerView<VH extends RecyclerView.ViewHolder> extends RecyclerView.Adapter<VH> implements RecyclerView.OnItemTouchListener {

    protected final BaseActivity mActivity;
    protected int mSelectedItem;

    private OnSelectClickListener mSelectListener;
    private GestureDetector mGestureDetector;

    protected AbstractRecyclerView(BaseActivity activity) {
        this.mActivity = activity;
    }

    public interface OnItemClickListener {
        public void onItemClick(View view, int position);
    }

    public interface OnSelectClickListener {
        public void onSelect(@NotNull View item, int position);

        public void onUnSelect(@Nullable View item, int position);
    }


    public void setOnSelectListener(Context context, OnSelectClickListener listener) {
        mSelectListener = listener;
        mGestureDetector = new GestureDetector(context, new GestureDetector.SimpleOnGestureListener() {
            @Override
            public boolean onSingleTapUp(MotionEvent e) {
                return true;
            }
        });
    }

    @Override
    public boolean onInterceptTouchEvent(RecyclerView view, MotionEvent e) {
        if (mSelectListener == null) return true;

        View item = view.findChildViewUnder(e.getX(), e.getY());
        if (item != null && mSelectListener != null && mGestureDetector.onTouchEvent(e)) {

            View prevView = findLastSelectedItem((RecyclerView) item.getParent());
            mSelectListener.onUnSelect(prevView, mSelectedItem);

            mSelectedItem = view.getChildPosition(item);
            mSelectListener.onSelect(item, mSelectedItem);

        }

        return false;
    }

    @Nullable
    private View findLastSelectedItem(RecyclerView rView) {

        LinearLayoutManager lm = (LinearLayoutManager) rView.getLayoutManager();

        View prevView = null;
        if (mSelectedItem > -1 &&
                mSelectedItem <= lm.findLastVisibleItemPosition() && mSelectedItem >= lm.findFirstVisibleItemPosition()) {

            int prevSelectedIndex = mSelectedItem - lm.findFirstVisibleItemPosition();
            prevView = rView.getChildAt(prevSelectedIndex);

            mSelectListener.onUnSelect(prevView, mSelectedItem);
        }

        return prevView;
    }

    @Override
    public void onTouchEvent(RecyclerView view, MotionEvent motionEvent) {
    }

    protected Resources getRes() {

        return mActivity.getResources();
    }

    protected int getColor(final int colorResId) {
        return getRes().getColor(colorResId);
    }
}