-------------------------------------------------
--- GET 10 FILES THAT REFERENCE THE SO ANSWER ---
-------------------------------------------------
SELECT
  MIN(id) as id,
  MIN(url) as url,
  line,
  MIN(CONCAT("https://github.com/", repo_name, "/blob/master/", path)) as link
FROM [snippets.java_a_q_distinct_url_file]
WHERE url="http://stackoverflow.com/a/7696791"
GROUP BY line
LIMIT 10;

=> a_7696791_refs.csv


--------------------------------------------------------------------
--- BUILD REGEX, VALIDATE WITH 10 SNIPPETS, AND SEARCH ON GITHUB ---
--------------------------------------------------------------------
SELECT
  id,
  CONCAT("https://github.com/", repo_name, "/blob/master/", path) as link,
  repo_name,
  path,
  REGEXP_MATCH(content, r'((?i:View[\s]*[^\(]+[\s]*\=[^;]+\.[\s]*getCurrentFocus[\s]*\([\s]*\)[^\{]+if[\s]*\([\s]*[^;]+[\s]*!=[\s]*null[\s]*\)[^\}]+InputMethodManager[\s]*[^=]+[\s]*=[\s]*\([\s]*InputMethodManager[\s]*\)[\s]*[^\}]*getSystemService[\s]*\([\s]*[^;]+[\s]*\.[\s]*INPUT_METHOD_SERVICE[\s]*\)[^\}]+\.hideSOftInputFromWindow[\s]*\([\s]*[^;]+[\s]*\.[\s]*getWindowToken[\s]*\([\s]*\)[\s]*,[^\)]+\)))')
  as matched_first,
  content
FROM (
  SELECT
    f.id as id,
    repo_name,
    path,
    content
  FROM (
    SELECT
      id,
      repo_name,
      path
    FROM
      [bigquery-public-data:github_repos.files]
    WHERE
      LOWER(path) LIKE '%.java'
  ) as f
  JOIN [bigquery-public-data:github_repos.contents] as c
  ON f.id = c.id
)
WHERE
  REGEXP_MATCH(content, r'((?i:View[\s]*[^\(]+[\s]*\=[^;]+\.[\s]*getCurrentFocus[\s]*\([\s]*\)[^\{]+if[\s]*\([\s]*[^;]+[\s]*!=[\s]*null[\s]*\)[^\}]+InputMethodManager[\s]*[^=]+[\s]*=[\s]*\([\s]*InputMethodManager[\s]*\)[\s]*[^\}]*getSystemService[\s]*\([\s]*[^;]+[\s]*\.[\s]*INPUT_METHOD_SERVICE[\s]*\)[^\}]+\.hideSOftInputFromWindow[\s]*\([\s]*[^;]+[\s]*\.[\s]*getWindowToken[\s]*\([\s]*\)[\s]*,[^\)]+\)))')
  OR REGEXP_MATCH(content, r'((?i:InputMethodManager[\s]*[^=]+[\s]*=[\s]*\([\s]*InputMethodManager[\s]*\)[\s]*[^\}]*getSystemService[\s]*\([^;]+[\s]*\.[\s]*INPUT_METHOD_SERVICE[\s]*\)[^\}]+View[\s]*[^=]+[\s]*=[\s]*[^;]+[\s]*\.[\s]*getCurrentFocus[\s]*\([\s]*\)[^\{]+if[\s]*\([\s]*[^;]+[\s]*==[\s]*null[\s]*\)[\s\S]+\.hideSOftInputFromWindow[\s]*\([\s]*[^;]+[\s]*\.getWindowToken[\s]*\([\s]*\)[\s]*,[^\)]+\)))');
  
=> java_top5_matches


-------------------------
--- NUMBER OF MATCHES ---
-------------------------
SELECT count(*)
FROM [snippets.java_top5_matches];

=> 1492


----------------------------------------
--- NUMBER OF DISTINCT MATCHED FILES ---
----------------------------------------
SELECT count(*)
FROM (
  SELECT id
  FROM [snippets.java_top5_matches]
  GROUP BY id
);

=> 402


---------------------------------------------------
--- EXTRACT MATCHES FROM DISTINCT MATCHED FILES ---
---------------------------------------------------
SELECT
  id,
  link,
  matched_first,
  CASE
    WHEN matched_first=1
    THEN REGEXP_EXTRACT(content, r'((?i:View[\s]*[^\(]+[\s]*\=[^;]+\.[\s]*getCurrentFocus[\s]*\([\s]*\)[^\{]+if[\s]*\([\s]*[^;]+[\s]*!=[\s]*null[\s]*\)[^\}]+InputMethodManager[\s]*[^=]+[\s]*=[\s]*\([\s]*InputMethodManager[\s]*\)[\s]*[^\}]*getSystemService[\s]*\([\s]*[^;]+[\s]*\.[\s]*INPUT_METHOD_SERVICE[\s]*\)[^\}]+\.hideSOftInputFromWindow[\s]*\([\s]*[^;]+[\s]*\.[\s]*getWindowToken[\s]*\([\s]*\)[\s]*,[^\)]+\)))')
    ELSE REGEXP_EXTRACT(content, r'((?i:InputMethodManager[\s]*[^=]+[\s]*=[\s]*\([\s]*InputMethodManager[\s]*\)[\s]*[^\}]*getSystemService[\s]*\([^;]+[\s]*\.[\s]*INPUT_METHOD_SERVICE[\s]*\)[^\}]+View[\s]*[^=]+[\s]*=[\s]*[^;]+[\s]*\.[\s]*getCurrentFocus[\s]*\([\s]*\)[^\{]+if[\s]*\([\s]*[^;]+[\s]*==[\s]*null[\s]*\)[\s\S]+\.hideSOftInputFromWindow[\s]*\([\s]*[^;]+[\s]*\.getWindowToken[\s]*\([\s]*\)[\s]*,[^\)]+\)))')
  END as match
FROM (
  SELECT
    distinct_files.id as id,
    distinct_files.link as link,
    all_files.content as content,
    all_files.matched_first as matched_first
  FROM (
    SELECT id, MIN(link) as link
    FROM [snippets.java_top5_matches]
    GROUP BY id
  ) as distinct_files
  LEFT JOIN [snippets.java_top5_matches] as all_files
  ON distinct_files.id = all_files.id
    AND distinct_files.link = all_files.link
  GROUP BY id, link, content, matched_first
);

=> java_top5_matches_distinct_files
 
 
-----------------------------------------------
--- EXTRACT URLS FOR DISTINCT MATCHED FILES ---
----------------------------------------------- 
SELECT
  id,
  link,
  CASE
    WHEN REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
    THEN REGEXP_EXTRACT(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
    WHEN REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
    THEN REGEXP_EXTRACT(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
    WHEN REGEXP_MATCH(url, r'https?:\/\/stackoverflow\.com\/questions\/[\d]+\/[^\s\/\#]+(?:\/|\#)([\d]+)')
    THEN CONCAT("http://stackoverflow.com/a/", REGEXP_EXTRACT(url, r'https?:\/\/stackoverflow\.com\/questions\/[\d]+\/[^\s\/\#]+(?:\/|\#)([\d]+)'))
    WHEN REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/questions\/[\d]+)')
    THEN CONCAT("http://stackoverflow.com/q/", REGEXP_EXTRACT(url, r'https?:\/\/stackoverflow\.com\/questions\/([\d]+)'))
    ELSE url END as url,
  line,
  match
FROM (
    SELECT
      id,
      link,
      REGEXP_REPLACE(
        REGEXP_EXTRACT(LOWER(line), r'(https?://stackoverflow\.com/[^\s)\.\"]*)'),
        r'(^https)',
        'http'
      ) as url,	
      line,
      match
    FROM (
      SELECT df.id as id, df.link as link, SPLIT(m.content, '\n') as line, df.match as match
      FROM [snippets.java_top5_matches_distinct_files] as df
      LEFT JOIN [snippets.java_top5_matches] as m
      ON df.id = m.id AND df.link = m.link
    )
    WHERE
      REGEXP_MATCH(line, r'(?i:https?://stackoverflow\.com/[^\s)\.\"]*)')
);
 
=> java_top5_matches_distinct_files_url
  

-----------------------------------------------------------------
--- TOTAL NUMBER OF FILES WITH REFERENCE TO Q OR A OF SNIPPET ---
-----------------------------------------------------------------
SELECT COUNT(id)
FROM (
  SELECT id
  FROM [snippets.java_a_q_distinct_url_file]
  WHERE url="http://stackoverflow.com/a/7696791"
    OR url=(
      SELECT url_q
      FROM [snippets.java_a_q_mapping]
      WHERE url_a="http://stackoverflow.com/a/7696791"
    )
  GROUP BY id
);

 => 34
  
 
-------------------------------------------------------------
--- SAVE MATCHED FILES WITH REFERENCE TO Q OR A OF SNIPPET ---
--------------------------------------------------------------
SELECT
    id, link, match
FROM
  [snippets.java_top5_matches_distinct_files]
WHERE
  id IN (
    SELECT
      id
    FROM
      [snippets.java_top5_matches_distinct_files_url]
    WHERE
      url="http://stackoverflow.com/a/7696791"
      OR url=(
        SELECT url_q
        FROM [snippets.java_a_q_mapping]
        WHERE url_a="http://stackoverflow.com/a/7696791"
      )
    GROUP BY id
  )
  
=> java_top5_matches_distinct_files_referenced
 
 
 -------------------------------------------------------------------
--- NUMBER OF MATCHED FILES WITH REFERENCE TO Q OR A OF SNIPPET ---
-------------------------------------------------------------------  
SELECT COUNT(*)
FROM [snippets.java_top5_matches_distinct_files_referenced];
 
 => 25
 
 
-------------------------------------------------------------
--- SAVE MATCHED FILES WITH REFERENCE TO Q OR A OF SNIPPET ---
--------------------------------------------------------------
SELECT
    id, link, match
FROM
  [snippets.java_top5_matches_distinct_files]
WHERE
  id NOT IN (
    SELECT
      id
    FROM
      [snippets.java_top5_matches_distinct_files_url]
    WHERE
      url="http://stackoverflow.com/a/7696791"
      OR url=(
        SELECT url_q
        FROM [snippets.java_a_q_mapping]
        WHERE url_a="http://stackoverflow.com/a/7696791"
      )
    GROUP BY id
  )
  
=> java_top5_matches_distinct_files_not_referenced
 
 
 ---------------------------------------------------------------------
--- NUMBER OF MATCHED FILES WITHOUT REFERENCE TO Q OR A OF SNIPPET ---
----------------------------------------------------------------------  
SELECT COUNT(*)
FROM [snippets.java_top5_matches_distinct_files_not_referenced];
 
=> 377
 
