    // utility method, adapted from http://stackoverflow.com/a/2581754
    public static <K, V extends Comparable<? super V>> Map<K, V> sortByValue(Map<K,V> map) {
        // make a List from the entry set to feed to Collections::sort
        List<Map.Entry<K, V>> list = new LinkedList<>(map.entrySet());

        // implement an anonymous Comparator to sort by the value of each Map.Entry
        Collections.sort(list, new Comparator<Map.Entry<K, V>>() {
            @Override
            public int compare(Map.Entry<K, V> o1, Map.Entry<K, V> o2) {
                // this sorts in reverse, because the max values are wanted
                return o2.getValue().compareTo(o1.getValue());
            }
        });

        // use a LinkedHashMap to preserve insertion order, not natural order
        Map<K, V> sorted = new LinkedHashMap<>();
        // can only use Map#putAll on another Map, so put Map#put each List entry
        for (Map.Entry<K, V> entry : list)
            sorted.put(entry.getKey(), entry.getValue());
        return sorted;
    }
