-------------------------------------------------
--- GET 10 FILES THAT REFERENCE THE SO ANSWER ---
-------------------------------------------------
SELECT
  MIN(id) as id,
  MIN(url) as url,
  line,
  MIN(CONCAT("https://github.com/", repo_name, "/blob/master/", path)) as link
FROM [snippets.java_a_q_distinct_url_file]
WHERE url="http://stackoverflow.com/a/2581754"
GROUP BY line
LIMIT 10;

=> a_2581754_refs.csv


--------------------------------------------------------------------
--- BUILD REGEX, VALIDATE WITH 10 SNIPPETS, AND SEARCH ON GITHUB ---
--------------------------------------------------------------------
SELECT
  id,
  CONCAT("https://github.com/", repo_name, "/blob/master/", path) as link,
  repo_name,
  path,
  content
FROM (
  SELECT
    f.id as id,
    repo_name,
    path,
    content
  FROM (
    SELECT
      id,
      repo_name,
      path
    FROM
      [bigquery-public-data:github_repos.files]
    WHERE
      LOWER(path) LIKE '%.java'
  ) as f
  JOIN [bigquery-public-data:github_repos.contents] as c
  ON f.id = c.id
)
WHERE
  REGEXP_MATCH(content, r'((?i:<[\s]*K[\s]*,[\s]*V[\s]+extends[\s]+Comparable[\s]*<[\s]*\?[\s]*super[\s]*V[\s]*>[\s]*>[\s]*Map[\s]*<[\s]*K[\s]*,[\s]*V[\s]*>[\s]*\w+[\s]*\([\s]*Map[\s]*<[\s]*K[\s]*,[\s]*V[\s]*>[\s]*[^\)]+\)[\s]*\{[\s\S]*List[\s]*<[\s]*Map[\s]*\.[\s]*Entry[\s]*<[\s]*K[\s]*,[\s]*V[\s]*>[\s]*>[\s]*[^=]+=[\s]*new[\s]*LinkedList[\s]*<[^;]*>[\s]*\([^;]+\.entrySet[\s]*\([\s]*\)[\s]*\);[^\}]*Collections[\s]*\.[\s]*sort[\s]*\([\s\S]+compareTo[\s]*\([^;]+\.[\s]*getValue[\s]*\([\s]*\)[\s]*\)[^;]*;[\s\S]*Map[\s]*\<K[\s]*,[\s]*V[\s]*>[^=]+\=[\s]*new[\s]*LinkedHashMap[\s]*<[^;]*>[\s]*\([\s]*\)[\s]*;[^\}]*for[\s]*\([\s]*Map[\s]*\.[\s]*Entry[\s]*<[\s]*K[\s]*,[\s]*V[\s]*>[^:]+:[^\{]+\)[\s]*[^;]+\.put[\s]*\([^,]+\.getKey[\s]*\([\s]*\)[\s]*,[^;]+\.getValue[\s]*\([\s]*\)[\s]*\)[\s]*;[\s]*\}*[\s]*return[^;]+;))');

=> java_top7_matches


-------------------------
--- NUMBER OF MATCHES ---
-------------------------
SELECT count(*)
FROM [snippets.java_top7_matches];

=> 160


----------------------------------------
--- NUMBER OF DISTINCT MATCHED FILES ---
----------------------------------------
SELECT count(*)
FROM (
  SELECT id
  FROM [snippets.java_top7_matches]
  GROUP BY id
);

=> 124


---------------------------------------------------
--- EXTRACT MATCHES FROM DISTINCT MATCHED FILES ---
---------------------------------------------------
SELECT
  id,
  link,
  REGEXP_EXTRACT(content, r'((?i:<[\s]*K[\s]*,[\s]*V[\s]+extends[\s]+Comparable[\s]*<[\s]*\?[\s]*super[\s]*V[\s]*>[\s]*>[\s]*Map[\s]*<[\s]*K[\s]*,[\s]*V[\s]*>[\s]*\w+[\s]*\([\s]*Map[\s]*<[\s]*K[\s]*,[\s]*V[\s]*>[\s]*[^\)]+\)[\s]*\{[\s\S]*List[\s]*<[\s]*Map[\s]*\.[\s]*Entry[\s]*<[\s]*K[\s]*,[\s]*V[\s]*>[\s]*>[\s]*[^=]+=[\s]*new[\s]*LinkedList[\s]*<[^;]*>[\s]*\([^;]+\.entrySet[\s]*\([\s]*\)[\s]*\);[^\}]*Collections[\s]*\.[\s]*sort[\s]*\([\s\S]+compareTo[\s]*\([^;]+\.[\s]*getValue[\s]*\([\s]*\)[\s]*\)[^;]*;[\s\S]*Map[\s]*\<K[\s]*,[\s]*V[\s]*>[^=]+\=[\s]*new[\s]*LinkedHashMap[\s]*<[^;]*>[\s]*\([\s]*\)[\s]*;[^\}]*for[\s]*\([\s]*Map[\s]*\.[\s]*Entry[\s]*<[\s]*K[\s]*,[\s]*V[\s]*>[^:]+:[^\{]+\)[\s]*[^;]+\.put[\s]*\([^,]+\.getKey[\s]*\([\s]*\)[\s]*,[^;]+\.getValue[\s]*\([\s]*\)[\s]*\)[\s]*;[\s]*\}*[\s]*return[^;]+;))')
  as match
FROM (
  SELECT
    distinct_files.id as id,
    distinct_files.link as link,
    all_files.content as content
  FROM (
    SELECT id, MIN(link) as link
    FROM [snippets.java_top7_matches]
    GROUP BY id
  ) as distinct_files
  LEFT JOIN [snippets.java_top7_matches] as all_files
  ON distinct_files.id = all_files.id
    AND distinct_files.link = all_files.link
  GROUP BY id, link, content
);

=> java_top7_matches_distinct_files
 
 
-----------------------------------------------
--- EXTRACT URLS FOR DISTINCT MATCHED FILES ---
----------------------------------------------- 
SELECT
  id,
  link,
  CASE
    WHEN REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
    THEN REGEXP_EXTRACT(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
    WHEN REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
    THEN REGEXP_EXTRACT(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
    WHEN REGEXP_MATCH(url, r'https?:\/\/stackoverflow\.com\/questions\/[\d]+\/[^\s\/\#]+(?:\/|\#)([\d]+)')
    THEN CONCAT("http://stackoverflow.com/a/", REGEXP_EXTRACT(url, r'https?:\/\/stackoverflow\.com\/questions\/[\d]+\/[^\s\/\#]+(?:\/|\#)([\d]+)'))
    WHEN REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/questions\/[\d]+)')
    THEN CONCAT("http://stackoverflow.com/q/", REGEXP_EXTRACT(url, r'https?:\/\/stackoverflow\.com\/questions\/([\d]+)'))
    ELSE url END as url,
  line,
  match
FROM (
    SELECT
      id,
      link,
      REGEXP_REPLACE(
        REGEXP_EXTRACT(LOWER(line), r'(https?://stackoverflow\.com/[^\s)\.\"]*)'),
        r'(^https)',
        'http'
      ) as url,	
      line,
      match
    FROM (
      SELECT df.id as id, df.link as link, SPLIT(m.content, '\n') as line, df.match as match
      FROM [snippets.java_top7_matches_distinct_files] as df
      LEFT JOIN [snippets.java_top7_matches] as m
      ON df.id = m.id AND df.link = m.link
    )
    WHERE
      REGEXP_MATCH(line, r'(?i:https?://stackoverflow\.com/[^\s)\.\"]*)')
);
 
=> java_top7_matches_distinct_files_url
  

-----------------------------------------------------------------
--- TOTAL NUMBER OF FILES WITH REFERENCE TO Q OR A OF SNIPPET ---
-----------------------------------------------------------------
SELECT COUNT(id)
FROM (
  SELECT id
  FROM [snippets.java_a_q_distinct_url_file]
  WHERE url="http://stackoverflow.com/a/2581754"
    OR url=(
      SELECT url_q
      FROM [snippets.java_a_q_mapping]
      WHERE url_a="http://stackoverflow.com/a/2581754"
    )
  GROUP BY id
);

=> 41
  
 
-------------------------------------------------------------
--- SAVE MATCHED FILES WITH REFERENCE TO Q OR A OF SNIPPET ---
--------------------------------------------------------------
SELECT
    id, link, match
FROM
  [snippets.java_top7_matches_distinct_files]
WHERE
  id IN (
    SELECT
      id
    FROM
      [snippets.java_top7_matches_distinct_files_url]
    WHERE
      url="http://stackoverflow.com/a/2581754"
      OR url=(
        SELECT url_q
        FROM [snippets.java_a_q_mapping]
        WHERE url_a="http://stackoverflow.com/a/2581754"
      )
    GROUP BY id
  );
  
=> java_top7_matches_distinct_files_referenced
 
 
 -------------------------------------------------------------------
--- NUMBER OF MATCHED FILES WITH REFERENCE TO Q OR A OF SNIPPET ---
-------------------------------------------------------------------  
SELECT COUNT(*)
FROM [snippets.java_top7_matches_distinct_files_referenced];
 
=> 12
 
 
-------------------------------------------------------------
--- SAVE MATCHED FILES WITH REFERENCE TO Q OR A OF SNIPPET ---
--------------------------------------------------------------
SELECT
    id, link, match
FROM
  [snippets.java_top7_matches_distinct_files]
WHERE
  id NOT IN (
    SELECT
      id
    FROM
      [snippets.java_top7_matches_distinct_files_url]
    WHERE
      url="http://stackoverflow.com/a/2581754"
      OR url=(
        SELECT url_q
        FROM [snippets.java_a_q_mapping]
        WHERE url_a="http://stackoverflow.com/a/2581754"
      )
    GROUP BY id
  );
  
=> java_top7_matches_distinct_files_not_referenced
 
 
 ---------------------------------------------------------------------
--- NUMBER OF MATCHED FILES WITHOUT REFERENCE TO Q OR A OF SNIPPET ---
----------------------------------------------------------------------  
SELECT COUNT(*)
FROM [snippets.java_top7_matches_distinct_files_not_referenced];
 
=> 112
 
